@@ -81,10 +81,10 @@ OperationParameters* AlgorithmMetricDilate::getParameters()
         
     ret->setHelpText(
         AString("For all metric vertices that are designated as bad, if they neighbor a non-bad vertex with data or are within the specified distance of such a vertex, ") +
-        "replace the value with a distance weighted average of nearby non-bad vertices that have data, otherwise set the value to zero.  " +
+        "replace the value with a distance-based weighted average of nearby non-bad vertices that have data, otherwise set the value to zero.  " +
         "No matter how small <distance> is, dilation will always use at least the immediate neighbor vertices.  " +
         "If -nearest is specified, it will use the value from the closest non-bad vertex with data within range instead of a weighted average.\n\n" +
-        "If -bad-vertex-roi is specified, only vertices with a positive value in the ROI are bad.  " +
+        "If -bad-vertex-roi is specified, all vertices with a positive ROI value are bad.  " +
         "If it is not specified, only vertices that have data, with a value of zero, are bad.  " +
         "If -data-roi is not specified, all vertices are assumed to have data.\n\n" +
         "Note that the -corrected-areas option uses an approximate correction for the change in distances along a group average surface.\n\n" +
@@ -194,6 +194,7 @@ AlgorithmMetricDilate::AlgorithmMetricDilate(ProgressObject* myProgObj, const Me
         myAreas = corrAreas->getValuePointerForColumn(0);
     }
     bool linear = (myMethod == LINEAR), nearest = (myMethod == NEAREST);
+    FastStatistics spacingStats;
     if (!linear && badNodeRoi != NULL)//if we know which nodes need to have their values replaced, then we can do the same thing at each vertex for each column
     {
         if (nearest)
@@ -202,6 +203,8 @@ AlgorithmMetricDilate::AlgorithmMetricDilate(ProgressObject* myProgObj, const Me
         } else {
             precomputeStencils(myStencils, mySurf, myAreas, badNodeRoi, dataRoi, corrAreas, distance, exponent, legacyCutoff);
         }
+    } else {
+        mySurf->getNodesSpacingStatistics(spacingStats);//use mean spacing to help set minimum stencil distance, since native surfaces might have a minimum of 0
     }
     if (columnNum == -1)
     {
@@ -213,7 +216,7 @@ AlgorithmMetricDilate::AlgorithmMetricDilate(ProgressObject* myProgObj, const Me
             myMetricOut->setColumnName(thisCol, myMetric->getColumnName(thisCol));
             if (badNodeRoi == NULL)
             {
-                processColumn(colScratch.data(), myInputData, mySurf, myAreas, badNodeRoi, dataRoi, corrAreas, distance, nearest, linear, exponent, legacyCutoff);
+                processColumn(colScratch.data(), myInputData, mySurf, myAreas, badNodeRoi, dataRoi, corrAreas, distance, nearest, linear, exponent, legacyCutoff, spacingStats.getMean());
             } else {
                 switch (myMethod)
                 {
@@ -224,7 +227,7 @@ AlgorithmMetricDilate::AlgorithmMetricDilate(ProgressObject* myProgObj, const Me
                         processColumn(colScratch.data(), numNodes, myInputData, myStencils);
                         break;
                     case LINEAR:
-                        processColumn(colScratch.data(), myInputData, mySurf, myAreas, badNodeRoi, dataRoi, corrAreas, distance, nearest, linear, exponent, legacyCutoff);
+                        processColumn(colScratch.data(), myInputData, mySurf, myAreas, badNodeRoi, dataRoi, corrAreas, distance, nearest, linear, exponent, legacyCutoff, spacingStats.getMean());
                         break;
                 }
             }
@@ -237,7 +240,7 @@ AlgorithmMetricDilate::AlgorithmMetricDilate(ProgressObject* myProgObj, const Me
         myMetricOut->setColumnName(0, myMetric->getColumnName(columnNum));
         if (badNodeRoi == NULL)
         {
-            processColumn(colScratch.data(), myInputData, mySurf, myAreas, badNodeRoi, dataRoi, corrAreas, distance, nearest, linear, exponent, legacyCutoff);
+            processColumn(colScratch.data(), myInputData, mySurf, myAreas, badNodeRoi, dataRoi, corrAreas, distance, nearest, linear, exponent, legacyCutoff, spacingStats.getMean());
         } else {
             switch (myMethod)
             {
@@ -248,7 +251,7 @@ AlgorithmMetricDilate::AlgorithmMetricDilate(ProgressObject* myProgObj, const Me
                     processColumn(colScratch.data(), numNodes, myInputData, myStencils);
                     break;
                 case LINEAR:
-                    processColumn(colScratch.data(), myInputData, mySurf, myAreas, badNodeRoi, dataRoi, corrAreas, distance, nearest, linear, exponent, legacyCutoff);
+                    processColumn(colScratch.data(), myInputData, mySurf, myAreas, badNodeRoi, dataRoi, corrAreas, distance, nearest, linear, exponent, legacyCutoff, spacingStats.getMean());
                     break;
             }
         }
@@ -306,11 +309,10 @@ void AlgorithmMetricDilate::processColumn(float* colScratch, const int& numNodes
 
 void AlgorithmMetricDilate::processColumn(float* colScratch, const float* myInputData, const SurfaceFile* mySurf, const float* myAreas,
                                           const MetricFile* badNodeRoi, const MetricFile* dataRoi, const MetricFile* corrAreas,
-                                          const float& distance, const bool& nearest, const bool& linear, const float& exponent, const bool legacyCutoff)
+                                          const float& distance, const bool& nearest, const bool& linear, const float& exponent, const bool legacyCutoff, const float meanSpacing)
 {
-    FastStatistics spacingStats;
-    mySurf->getNodesSpacingStatistics(spacingStats);//use mean spacing to help set minimum stencil distance, since native surfaces might have a minimum of 0
-    float cutoffBase = max(2.0f * distance, 2.0f * spacingStats.getMean()), cutoffRatio = max(1.1f, pow(49.0f, 1.0f / (exponent - 2.0f)));//find what ratio from closest vertex corresponds to having 98% of total weight accounted for on a plane, assuming non-adverse ROI
+    float cutoffBase = max(2.0f * distance, 2.0f * meanSpacing), cutoffRatio = max(1.1f, pow(49.0f, 1.0f / (exponent - 2.0f)));//find what ratio from closest vertex corresponds to having 98% of total weight accounted for on a plane, assuming non-adverse ROI
+    float minKernel = 1.5f * meanSpacing;//small kernels are cheap for weighted, use similar minimum distance as volume dilate
     float legacyCutoffRatio = 1.5f, test = pow(10.0f, 1.0f / exponent);//old logic: find what cutoff ratio corresponds to a tenth of weight, but don't use more than a 1.5 * nearest cutoff
     if (test > 1.0f && test < legacyCutoffRatio)//if it is less than 1, the exponent is weird, so simply ignore it and use default
     {//this generally cut off early, causing the result to behave like a higher exponent was used
@@ -322,56 +324,26 @@ void AlgorithmMetricDilate::processColumn(float* colScratch, const float* myInpu
         }
     }
     int numNodes = mySurf->getNumberOfNodes();
-    vector<char> charRoi(numNodes);
+    vector<char> charRoi(numNodes, 0);
     const float* badRoiData = NULL;
     if (badNodeRoi != NULL) badRoiData = badNodeRoi->getValuePointerForColumn(0);
     const float* dataRoiVals = NULL;
     if (dataRoi != NULL)
     {
         dataRoiVals = dataRoi->getValuePointerForColumn(0);
-        if (badRoiData != NULL)
+    }
+    for (int i = 0; i < numNodes; ++i)
+    {
+        if (badRoiData == NULL)
         {
-            for (int i = 0; i < numNodes; ++i)
+            if ((dataRoiVals == NULL || dataRoiVals[i] > 0.0f) && myInputData[i] != 0.0f)
             {
-                if (dataRoiVals[i] > 0.0f && !(badRoiData[i] > 0.0f))//"not greater than" to trap NaNs
-                {
-                    charRoi[i] = 1;
-                } else {
-                    charRoi[i] = 0;
-                }
+                charRoi[i] = 1;
             }
         } else {
-            for (int i = 0; i < numNodes; ++i)
+            if ((dataRoiVals == NULL || dataRoiVals[i] > 0.0f) && !(badRoiData[i] > 0.0f))//"not greater than" to trap NaNs
             {
-                if (dataRoiVals[i] > 0.0f && myInputData[i] != 0.0f)
-                {
-                    charRoi[i] = 1;
-                } else {
-                    charRoi[i] = 0;
-                }
-            }
-        }
-    } else {
-        if (badRoiData != NULL)
-        {
-            for (int i = 0; i < numNodes; ++i)
-            {
-                if (!(badRoiData[i] > 0.0f))//ditto
-                {
-                    charRoi[i] = 1;
-                } else {
-                    charRoi[i] = 0;
-                }
-            }
-        } else {
-            for (int i = 0; i < numNodes; ++i)
-            {
-                if (myInputData[i] != 0.0f)
-                {
-                    charRoi[i] = 1;
-                } else {
-                    charRoi[i] = 0;
-                }
+                charRoi[i] = 1;
             }
         }
     }
@@ -400,7 +372,7 @@ void AlgorithmMetricDilate::processColumn(float* colScratch, const float* myInpu
             } else {
                 badNode = (myInputData[i] == 0.0f);
             }
-            if ((dataRoiVals == NULL || dataRoiVals[i] > 0.0f) && badNode)
+            if (badNode)
             {
                 float closestDist;//NOTE: the only time this function is called with a badRoi is when using linear, which doesn't use the closest distance
                 int closestNode = myGeoHelp->getClosestNodeInRoi(i, charRoi.data(), distance, closestDist);
@@ -511,7 +483,7 @@ void AlgorithmMetricDilate::processColumn(float* colScratch, const float* myInpu
                             } else {
                                 if (exponent > 2.0f && cutoffRatio < 100.0f && cutoffRatio > 1.0f)//if the ratio is sane, use it, but never exceed cutoffBase
                                 {
-                                    cutoffDist = max(min(cutoffRatio * closestDist, cutoffDist), cutoffBase * 0.25f);//but small kernels are rather cheap anyway, so have a minimum size just in case
+                                    cutoffDist = max(min(cutoffRatio * closestDist, cutoffDist), minKernel);//but small kernels are rather cheap anyway, so have a minimum size just in case
                                 }
                             }
                             myGeoHelp->getNodesToGeoDist(i, cutoffDist, nodeList, distList);
@@ -559,6 +531,7 @@ void AlgorithmMetricDilate::precomputeStencils(vector<pair<int, StencilElem> >&
     FastStatistics spacingStats;
     mySurf->getNodesSpacingStatistics(spacingStats);//use mean spacing to help set minimum stencil distance, since native surfaces might have a minimum of 0
     float cutoffBase = max(2.0f * distance, 2.0f * spacingStats.getMean()), cutoffRatio = max(1.1f, pow(49.0f, 1.0f / (exponent - 2.0f)));//find what ratio from closest vertex corresponds to having 98% of total weight accounted for on a plane, assuming non-adverse ROI
+    float minKernel = 1.5f * spacingStats.getMean();//small kernels are cheap for weighted, use similar minimum distance as volume dilate
     float legacyCutoffRatio = 1.5f, test = pow(10.0f, 1.0f / exponent);//old logic: find what cutoff ratio corresponds to a tenth of weight, but don't use more than a 1.5 * nearest cutoff
     if (test > 1.0f && test < legacyCutoffRatio)//if it is less than 1, the exponent is weird, so simply ignore it and use default
     {//this generally cut off early, causing the result to behave like a higher exponent was used
@@ -570,39 +543,22 @@ void AlgorithmMetricDilate::precomputeStencils(vector<pair<int, StencilElem> >&
         }
     }
     int numNodes = mySurf->getNumberOfNodes();
-    vector<char> charRoi(numNodes);
+    vector<char> charRoi(numNodes, 0);
     const float* dataRoiVals = NULL;
     int badCount = 0;
     if (dataRoi != NULL)
     {
         dataRoiVals = dataRoi->getValuePointerForColumn(0);
-        for (int i = 0; i < numNodes; ++i)
+    }
+    for (int i = 0; i < numNodes; ++i)
+    {
+        if (badNodeData[i] > 0.0f)
         {
-            if (!(badNodeData[i] > 0.0f))//in case some clown uses NaN in the ROI instead of 0
-            {
-                if (dataRoiVals[i] > 0.0f)
-                {
-                    charRoi[i] = 1;
-                } else {
-                    charRoi[i] = 0;
-                }
-            } else {
-                if (dataRoiVals[i] > 0.0f)
-                {
-                    ++badCount;
-                }
-                charRoi[i] = 0;
-            }
-        }
-    } else {
-        for (int i = 0; i < numNodes; ++i)
-        {
-            if (!(badNodeData[i] > 0.0f))
+            ++badCount;
+        } else {
+            if (dataRoiVals == NULL || dataRoiVals[i] > 0.0f)
             {
                 charRoi[i] = 1;
-            } else {
-                ++badCount;
-                charRoi[i] = 0;
             }
         }
     }
@@ -626,7 +582,7 @@ void AlgorithmMetricDilate::precomputeStencils(vector<pair<int, StencilElem> >&
 #pragma omp CARET_FOR schedule(dynamic)
         for (int i = 0; i < numNodes; ++i)
         {
-            if (badNodeData[i] > 0.0f && (dataRoiVals == NULL || dataRoiVals[i] > 0.0f))
+            if (badNodeData[i] > 0.0f)
             {
                 int myIndex;
 #pragma omp critical
@@ -664,7 +620,7 @@ void AlgorithmMetricDilate::precomputeStencils(vector<pair<int, StencilElem> >&
                     } else {
                         if (exponent > 2.0f && cutoffRatio < 100.0f && cutoffRatio > 1.0f)//if the ratio is sane, use it, but never exceed cutoffBase
                         {
-                            cutoffDist = max(min(cutoffRatio * closestDist, cutoffDist), cutoffBase * 0.25f);//but small kernels are rather cheap anyway, so have a minimum size just in case
+                            cutoffDist = max(min(cutoffRatio * closestDist, cutoffDist), minKernel);//but small kernels are rather cheap anyway, so have a minimum size just in case
                         }
                     }
                     myGeoHelp->getNodesToGeoDist(i, cutoffDist, nodeList, distList);
@@ -703,39 +659,22 @@ void AlgorithmMetricDilate::precomputeNearest(vector<pair<int, int> >& myNearest
     CaretAssert(badNodeRoi != NULL);//because it should never be called if we don't know exactly what nodes we are replacing
     const float* badNodeData = badNodeRoi->getValuePointerForColumn(0);
     int numNodes = mySurf->getNumberOfNodes();
-    vector<char> charRoi(numNodes);
+    vector<char> charRoi(numNodes, 0);
     const float* dataRoiVals = NULL;
     int badCount = 0;
     if (dataRoi != NULL)
     {
         dataRoiVals = dataRoi->getValuePointerForColumn(0);
-        for (int i = 0; i < numNodes; ++i)
+    }
+    for (int i = 0; i < numNodes; ++i)
+    {
+        if (badNodeData[i] > 0.0f)
         {
-            if (!(badNodeData[i] > 0.0f))//in case some clown uses NaN as "bad" in the ROI
-            {
-                if (dataRoiVals[i] > 0.0f)
-                {
-                    charRoi[i] = 1;
-                } else {
-                    charRoi[i] = 0;
-                }
-            } else {
-                if (dataRoiVals[i] > 0.0f)
-                {
-                    ++badCount;
-                }
-                charRoi[i] = 0;
-            }
-        }
-    } else {
-        for (int i = 0; i < numNodes; ++i)
-        {
-            if (!(badNodeData[i] > 0.0f))
+            ++badCount;
+        } else {
+            if (dataRoiVals == NULL || dataRoiVals[i] > 0.0f)
             {
                 charRoi[i] = 1;
-            } else {
-                ++badCount;
-                charRoi[i] = 0;
             }
         }
     }
@@ -759,7 +698,7 @@ void AlgorithmMetricDilate::precomputeNearest(vector<pair<int, int> >& myNearest
 #pragma omp CARET_FOR schedule(dynamic)
         for (int i = 0; i < numNodes; ++i)
         {
-            if (badNodeData[i] > 0.0f && (dataRoiVals == NULL || dataRoiVals[i] > 0.0f))
+            if (badNodeData[i] > 0.0f)
             {
                 int myIndex;
 #pragma omp critical
