@@ -37,6 +37,7 @@
 #include "BrainOpenGLTextRenderInterface.h"
 #include "CaretAssert.h"
 #include "CaretException.h"
+#include "DisplayPropertiesAnnotationTextSubstitution.h"
 #include "EventGetViewportSize.h"
 #include "EventManager.h"
 
@@ -541,6 +542,7 @@ AnnotationArrangerExecutor::getAnnotationsForArranging(const AnnotationArrangerI
     annotationsOut.clear();
     
     std::vector<AnnotationCoordinateSpaceEnum::Enum> spaces;
+    spaces.push_back(AnnotationCoordinateSpaceEnum::SPACER);
     spaces.push_back(AnnotationCoordinateSpaceEnum::TAB);
     spaces.push_back(AnnotationCoordinateSpaceEnum::WINDOW);
     
@@ -615,6 +617,10 @@ AnnotationArrangerExecutor::setupAnnotationInfo(const AnnotationArrangerInputs&
             case AnnotationCoordinateSpaceEnum::CHART:
                 CaretAssert(0);
                 break;
+            case AnnotationCoordinateSpaceEnum::SPACER:
+                getSpacerTabViewport(annotation->getSpacerTabIndex(),
+                                     annViewport);
+                break;
             case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
                 CaretAssert(0);
                 break;
@@ -698,6 +704,7 @@ AnnotationArrangerExecutor::setupAnnotationInfo(const AnnotationArrangerInputs&
                 const AnnotationText* textAnn = dynamic_cast<const AnnotationText*>(twoDimAnn);
                 CaretAssert(textAnn);
                 arrangerInputs.getTextRender()->getBoundsForTextAtViewportCoords(*textAnn,
+                                                                                 arrangerInputs.getDrawingFlags(),
                                                                                  viewportPixelOneXYZ[0],
                                                                                  viewportPixelOneXYZ[1],
                                                                                  viewportPixelOneXYZ[2],
@@ -862,7 +869,9 @@ void
 AnnotationArrangerExecutor::getTabViewport(const int32_t tabIndex,
                                            int32_t tabViewportOut[4])
 {
-    
+    /*
+     * Check cached viewports to avoid retrieving it a second time
+     */
     std::map<int32_t, ViewportArray>::iterator iter = m_tabViewports.find(tabIndex);
     if (iter != m_tabViewports.end()) {
         tabViewportOut[0] = iter->second.m_viewport[0];
@@ -891,6 +900,31 @@ AnnotationArrangerExecutor::getTabViewport(const int32_t tabIndex,
     }
 }
 
+/**
+ * Get the viewport for the given spacer tab.  Viewports are cached for efficiency.
+ *
+ * @param spacerTabIndex
+ *     Index of the spacer tab.
+ * @param tabViewportOut
+ *     Viewport of the tab.
+ * @throw  CaretException
+ *     If there is an error.
+ */
+void
+AnnotationArrangerExecutor::getSpacerTabViewport(const SpacerTabIndex& spacerTabIndex,
+                                                 int32_t tabViewportOut[4])
+{
+    EventGetViewportSize vpEvent(spacerTabIndex);
+    EventManager::get()->sendEvent(vpEvent.getPointer());
+    if (vpEvent.isViewportSizeValid()) {
+        vpEvent.getViewportSize(tabViewportOut);
+    }
+    else {
+        throw CaretException("Failed to get viewport size for spacer tab index "
+                             + spacerTabIndex.toString());
+    }
+}
+
 /**
  * Get a description of this object's content.
  * @return String describing this object's content.
