@@ -38,6 +38,7 @@
 
 namespace caret {
     class AnnotationFile;
+    class AnnotationTextSubstitutionFile;
     class AnnotationManager;
     class Border;
     class BorderFile;
@@ -69,6 +70,7 @@ namespace caret {
     class CiftiScalarDataSeriesFile;
     class DisplayProperties;
     class DisplayPropertiesAnnotation;
+    class DisplayPropertiesAnnotationTextSubstitution;
     class DisplayPropertiesBorders;
     class DisplayPropertiesFiberOrientation;
     class DisplayPropertiesFoci;
@@ -84,6 +86,7 @@ namespace caret {
     class ImageFile;
     class LabelFile;
     class MetricFile;
+    class MetricDynamicConnectivityFile;
     class ModelChart;
     class ModelChartTwo;
     class ModelSurfaceMontage;
@@ -92,12 +95,14 @@ namespace caret {
     class PaletteFile;
     class RgbaFile;
     class SceneClassAssistant;
+    class Scene;
     class SceneFile;
     class SelectionManager;
     class SpecFile;
     class Surface;
     class SurfaceFile;
     class SurfaceProjectedItem;
+    class VolumeDynamicConnectivityFile;
     class VolumeFile;
     
     class Brain : public CaretObject, public EventListenerInterface, public SceneableInterface {
@@ -137,6 +142,8 @@ namespace caret {
         
         const AnnotationFile* getSceneAnnotationFile() const;
         
+        void getAnnotationTextSubstitutionFiles(std::vector<AnnotationTextSubstitutionFile*>& annSubFilesOut) const;
+        
         int32_t getNumberOfFociFiles() const;
         
         FociFile* getFociFile(const int32_t indx);
@@ -183,6 +190,10 @@ namespace caret {
         
         const VolumeFile* getVolumeFile(const int32_t volumeFileIndex) const;
         
+        void getVolumeDynamicConnectivityFiles(std::vector<VolumeDynamicConnectivityFile*>& volumeDynamicConnectivityFilesOut) const;
+        
+        void getMetricDynamicConnectivityFiles(std::vector<MetricDynamicConnectivityFile*>& metricDynamicConnectivityFilesOut) const;
+        
         void resetBrain();
         
         void resetBrainKeepSceneFiles();
@@ -371,6 +382,10 @@ namespace caret {
         
         const DisplayPropertiesAnnotation* getDisplayPropertiesAnnotation() const;
         
+        DisplayPropertiesAnnotationTextSubstitution* getDisplayPropertiesAnnotationTextSubstitution();
+        
+        const DisplayPropertiesAnnotationTextSubstitution* getDisplayPropertiesAnnotationTextSubstitution() const;
+        
         DisplayPropertiesBorders* getDisplayPropertiesBorders();
         
         const DisplayPropertiesBorders* getDisplayPropertiesBorders() const;
@@ -428,6 +443,10 @@ namespace caret {
         
         const GapsAndMargins* getGapsAndMargins() const;
         
+        bool isSurfaceMatchingToAnatomical() const;
+        
+        void setSurfaceMatchingToAnatomical(const bool matchStatus);
+        
     private:
         /**
          * Reset the brain scene file mode
@@ -619,6 +638,10 @@ namespace caret {
                                    CaretDataFile* caretDataFile,
                                    const AString& filename);
         
+        AnnotationTextSubstitutionFile* addReadOrReloadAnnotationTextSubstitutionFile(const FileModeAddReadReload fileMode,
+                                                                              CaretDataFile* caretDataFile,
+                                                                              const AString& filename);
+        
         BorderFile* addReadOrReloadBorderFile(const FileModeAddReadReload fileMode,
                                               CaretDataFile* caretDataFile,
                                               const AString& filename);
@@ -695,6 +718,8 @@ namespace caret {
         
         void initializeDenseDataSeriesFile(CiftiBrainordinateDataSeriesFile* dataSeriesFile);
         
+        void initializeVolumeFile(VolumeFile* volumeFile);
+        
         void updateChartModel();
         
         void updateVolumeSliceModel();
@@ -719,6 +744,8 @@ namespace caret {
         
         AnnotationFile* m_sceneAnnotationFile;
         
+        std::vector<AnnotationTextSubstitutionFile*> m_annotationSubstitutionFiles;
+        
         std::vector<BorderFile*> m_borderFiles;
         
         std::vector<FociFile*> m_fociFiles;
@@ -810,6 +837,12 @@ namespace caret {
          */
         DisplayPropertiesAnnotation* m_displayPropertiesAnnotation;
         
+        /**
+         * Display properties for annotation text substitutions - DO NOT delete since this
+         * is also in the displayProperties std::vector.
+         */
+        DisplayPropertiesAnnotationTextSubstitution* m_displayPropertiesAnnotationTextSubstitution;
+        
         /**
          * Display properties for borders - DO NOT delete since this
          * is also in the displayProperties std::vector.
@@ -848,6 +881,10 @@ namespace caret {
         std::map<DataFileTypeEnum::Enum, int32_t> m_duplicateFileNameCounter;
         
         GapsAndMargins* m_gapsAndMargins;
+        
+        Scene* m_activeScene = NULL;
+        
+        bool m_surfaceMatchingToAnatomicalFlag = false;
     };
 
 } // namespace
