@@ -26,10 +26,13 @@
 
 #include "AnnotationCoordinateSpaceEnum.h"
 #include "AnnotationSizingHandleTypeEnum.h"
+#include "AnnotationSurfaceOffsetVectorTypeEnum.h"
 #include "BrainOpenGLFixedPipeline.h"
+#include "BrainOpenGLTextRenderInterface.h"
 #include "CaretObject.h"
 #include "CaretOpenGLInclude.h"
 #include "Plane.h"
+#include "SpacerTabIndex.h"
 
 
 namespace caret {
@@ -41,6 +44,7 @@ namespace caret {
     class AnnotationFile;
     class AnnotationImage;
     class AnnotationLine;
+    class AnnotationOneDimensionalShape;
     class AnnotationOval;
     class AnnotationText;
     class AnnotationTwoDimensionalShape;
@@ -64,13 +68,17 @@ namespace caret {
                    const float centerToEyeDistance,
                    const int32_t windowIndex,
                    const int32_t tabIndex,
-                   const WindowDrawingMode windowDrawingMode)
+                   const SpacerTabIndex &spacerTabIndex,
+                   const WindowDrawingMode windowDrawingMode,
+                   const bool annotationUserInputModeFlag)
             : m_brain(brain),
             m_drawingMode(drawingMode),
             m_centerToEyeDistance(centerToEyeDistance),
             m_windowIndex(windowIndex),
             m_tabIndex(tabIndex),
-            m_windowDrawingMode(windowDrawingMode) {
+            m_spacerTabIndex(spacerTabIndex),
+            m_windowDrawingMode(windowDrawingMode),
+            m_annotationUserInputModeFlag(annotationUserInputModeFlag) {
             }
             
             Brain* m_brain;
@@ -78,7 +86,9 @@ namespace caret {
             const float m_centerToEyeDistance;
             const int32_t m_windowIndex;
             const int32_t m_tabIndex;
+            const SpacerTabIndex m_spacerTabIndex;
             const WindowDrawingMode m_windowDrawingMode;
+            const bool m_annotationUserInputModeFlag;
         };
         
         BrainOpenGLAnnotationDrawingFixedPipeline(BrainOpenGLFixedPipeline* brainOpenGLFixedPipeline);
@@ -89,7 +99,8 @@ namespace caret {
                              const AnnotationCoordinateSpaceEnum::Enum drawingCoordinateSpace,
                              std::vector<AnnotationColorBar*>& colorBars,
                              std::vector<Annotation*>& notInFileAnnotations,
-                             const Surface* surfaceDisplayed);
+                             const Surface* surfaceDisplayed,
+                             const float surfaceViewScaling);
 
         void drawModelSpaceAnnotationsOnVolumeSlice(Inputs* inputs,
                                                     const Plane& plane,
@@ -185,6 +196,14 @@ namespace caret {
                             Annotation* annotation,
                             const Surface* surfaceDisplayed);
         
+        bool drawTwoDimAnnotationSurfaceTextureOffset(AnnotationFile* annotationFile,
+                                                      AnnotationTwoDimensionalShape* annotation,
+                                                      const Surface* surfaceDisplayed);
+        
+        bool drawOneDimAnnotationSurfaceTextureOffset(AnnotationFile* annotationFile,
+                                                      AnnotationOneDimensionalShape* annotation,
+                                                      const Surface* surfaceDisplayed);
+        
         void drawColorBar(AnnotationFile* annotationFile,
                           AnnotationColorBar* colorBar);
         
@@ -192,22 +211,48 @@ namespace caret {
                      AnnotationBox* box,
                        const Surface* surfaceDisplayed);
         
+        bool drawBoxSurfaceTangentOffset(AnnotationFile* annotationFile,
+                                         AnnotationBox* box,
+                                         const float surfaceExtentZ,
+                                         const float vertexXYZ[3]);
+
         bool drawImage(AnnotationFile* annotationFile,
                        AnnotationImage* image,
-                      const Surface* surfaceDisplayed);
+                       const Surface* surfaceDisplayed);
+        
+        bool drawImageSurfaceTangentOffset(AnnotationFile* annotationFile,
+                                           AnnotationImage* image,
+                                           const float surfaceExtentZ,
+                                           const float vertexXYZ[3]);
         
         bool drawLine(AnnotationFile* annotationFile,
                       AnnotationLine* line,
                       const Surface* surfaceDisplayed);
         
+        bool drawLineSurfaceTextureOffset(AnnotationFile* annotationFile,
+                                          AnnotationLine* line,
+                                          const Surface* surfaceDisplayed,
+                                          const float surfaceExtentZ);
+        
         bool drawOval(AnnotationFile* annotationFile,
                       AnnotationOval* oval,
                       const Surface* surfaceDisplayed);
         
+        bool drawOvalSurfaceTangentOffset(AnnotationFile* annotationFile,
+                                          AnnotationOval* oval,
+                                          const float surfaceExtentZ,
+                                          const float vertexXYZ[3]);
+        
         bool drawText(AnnotationFile* annotationFile,
                       AnnotationText* text,
                        const Surface* surfaceDisplayed);
         
+        bool drawTextSurfaceTangentOffset(AnnotationFile* annotationFile,
+                                          AnnotationText* text,
+                                          const float surfaceExtentZ,
+                                          const float vertexXYZ[3],
+                                          const float vertexNormalXYZ[3]);
+        
         void drawColorBarSections(const AnnotationColorBar* colorBar,
                                   const float bottomLeft[3],
                                   const float bottomRight[3],
@@ -216,12 +261,12 @@ namespace caret {
                                   const float sectionsHeightInPixels);
         
         void drawColorBarText(const AnnotationColorBar* colorBar,
-                              const float bottomLeft[3],
-                              const float bottomRight[3],
-                              const float topRight[3],
-                              const float topLeft[3],
-                              const float textHeightInPixels,
-                              const float offsetFromTopInPixels);
+                               const float bottomLeft[3],
+                               const float bottomRight[3],
+                               const float topRight[3],
+                               const float topLeft[3],
+                               const float textHeightInPixels,
+                               const float offsetFromTopInPixels);
 
         void drawColorBarTickMarks(const AnnotationColorBar* colorBar,
                                    const float bottomLeft[3],
@@ -283,14 +328,7 @@ namespace caret {
                                                        const float topLeft[3],
                                                        std::vector<float>& lineCoordinatesOut,
                                                        std::vector<float>& arrowCoordinatesOut) const;
-        
-        static void expandBox(float bottomLeft[3],
-                       float bottomRight[3],
-                       float topRight[3],
-                       float topLeft[3],
-                       const float extraSpaceX,
-                       const float extraSpaceY);
-        
+                
         void setSelectionBoxColor();
         
         void startOpenGLForDrawing(GLint* savedShadeModelOut,
@@ -310,10 +348,24 @@ namespace caret {
         
         float getLineWidthFromPercentageHeight(const float percentageHeight) const;
         
+        float getLineWidthPercentageInSelectionMode(const Annotation* annotation) const;
+        
+        float estimateColorBarWidth(const AnnotationColorBar* colorBar,
+                                    const float textHeightInPixels) const;
+        
+        void getSurfaceNormalVector(const Surface* surfaceDisplayed,
+                                    const int32_t vertexIndex,
+                                    float normalVectorOut[3]) const;
+        
+        bool isBackFacing(const float xyz[3],
+                          const float normal[3]) const;
+        
         BrainOpenGLFixedPipeline* m_brainOpenGLFixedPipeline;
         
         Inputs* m_inputs;
         
+        float m_surfaceViewScaling;
+        
         /**
          * Dummy annotation file is used for annotations that 
          * do not belong to a file.  This includes the 
@@ -350,16 +402,20 @@ namespace caret {
         
         float m_lineWidthMinimum = 1.0f;
         
+        BrainOpenGLTextRenderInterface::DrawingFlags m_textDrawingFlags;
+        
         std::unique_ptr<EventOpenGLObjectToWindowTransform> m_transformEvent;
         
-        static constexpr float s_sizingHandleLineWidthInPixels = 2.0f;
+        static const float s_sizingHandleLineWidthInPixels;
+        static const float s_selectionLineMinimumPixelWidth;
         
         // ADD_NEW_MEMBERS_HERE
 
     };
     
 #ifdef __BRAIN_OPEN_G_L_ANNOTATION_DRAWING_FIXED_PIPELINE_DECLARE__
-    // <PLACE DECLARATIONS OF STATIC MEMBERS HERE>
+    const float BrainOpenGLAnnotationDrawingFixedPipeline::s_sizingHandleLineWidthInPixels  = 2.0f;
+    const float BrainOpenGLAnnotationDrawingFixedPipeline::s_selectionLineMinimumPixelWidth = 5.0f;
 #endif // __BRAIN_OPEN_G_L_ANNOTATION_DRAWING_FIXED_PIPELINE_DECLARE__
 
 } // namespace
