@@ -32,7 +32,7 @@
 
 
 namespace caret {
-
+    
     class Brain;
     class BrowserTabContent;
     class CiftiMappableDataFile;
@@ -46,6 +46,15 @@ namespace caret {
     class BrainOpenGLVolumeSliceDrawing : public CaretObject {
         
     public:
+        /**
+         * Indicates drawing volume sclice "ALL that shows
+         * axial, coronal, and parasagittal at same time
+         */
+        enum class AllSliceViewMode {
+            ALL_YES,
+            ALL_NO
+        };
+        
         BrainOpenGLVolumeSliceDrawing();
         
         virtual ~BrainOpenGLVolumeSliceDrawing();
@@ -56,9 +65,26 @@ namespace caret {
                   const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
                   const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
                   const int32_t viewport[4]);
-
+        
+        static void setOrthographicProjection(const AllSliceViewMode allSliceViewMode,
+                                              const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                              const BoundingBox& boundingBox,
+                                              const float zoomFactor,
+                                              const int viewport[4],
+                                              double orthographicBoundsOut[6]);
+        
+        static void drawSurfaceOutline(const VolumeMappableInterface* underlayVolume,
+                                       const ModelTypeEnum::Enum modelType,
+                                       const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
+                                       const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                       const float sliceXYZ[3],
+                                       const Plane& plane,
+                                       VolumeSurfaceOutlineSetModel* outlineSet,
+                                       BrainOpenGLFixedPipeline* fixedPipelineDrawing,
+                                       const bool useNegativePolygonOffsetFlag);
+        
         // ADD_NEW_METHODS_HERE
-
+        
     private:
         /**
          * Holds values in the slice for a volume so that they
@@ -171,7 +197,7 @@ namespace caret {
         };
         
         BrainOpenGLVolumeSliceDrawing(const BrainOpenGLVolumeSliceDrawing&);
-
+        
         BrainOpenGLVolumeSliceDrawing& operator=(const BrainOpenGLVolumeSliceDrawing&);
         
         void drawVolumeSlicesForAllStructuresView(const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
@@ -183,34 +209,37 @@ namespace caret {
                                       const VolumeSliceViewAllPlanesLayoutEnum::Enum allPlanesLayout,
                                       const int32_t viewport[4]);
         
-        void drawVolumeSliceViewType(const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
-                           const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
-                           const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                           const int32_t viewport[4]);
+        void drawVolumeSliceViewType(const AllSliceViewMode allSliceViewMode,
+                                     const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
+                                     const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
+                                     const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                     const int32_t viewport[4]);
         
-        void drawVolumeSliceViewTypeMontage(const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
+        void drawVolumeSliceViewTypeMontage(const AllSliceViewMode allSliceViewMode,
+                                            const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
                                             const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
-                                  const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                                  const int32_t viewport[4]);
+                                            const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                            const int32_t viewport[4]);
         
-        void drawVolumeSliceViewProjection(const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
+        void drawVolumeSliceViewProjection(const AllSliceViewMode allSliceViewMode,
+                                           const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
                                            const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
-                                 const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                                 const float sliceCoordinates[3],
-                                 const int32_t viewport[4]);
-        
-        void drawObliqueSlice(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                              Matrix4x4& transformationMatrix,
-                              const Plane& plane);
+                                           const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                           const float sliceCoordinates[3],
+                                           const int32_t viewport[4]);
         
         void drawOrthogonalSlice_LPI_ONLY(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                                 const float sliceCoordinates[3],
-                                 const Plane& plane);
+                                          const float sliceCoordinates[3],
+                                          const Plane& plane);
         
         void drawOrthogonalSlice(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
                                  const float sliceCoordinates[3],
                                  const Plane& plane);
         
+        void drawOrthogonalSliceAllView(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                        const float sliceCoordinates[3],
+                                        const Plane& plane);
+        
         void drawOrthogonalSliceWithCulling(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
                                             const float sliceCoordinates[3],
                                             const Plane& plane);
@@ -220,12 +249,11 @@ namespace caret {
                                       const float sliceCoordinates[3],
                                       Plane& planeOut);
         
-        void drawAxesCrosshairsOrthoAndOblique(const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
-                                               const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                                               const float sliceCoordinates[3],
-                                               const bool drawCrosshairsFlag,
-                                               const bool drawCrosshairLabelsFlag);
-        
+        void drawAxesCrosshairsOrtho(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                     const float sliceCoordinates[3],
+                                     const bool drawCrosshairsFlag,
+                                     const bool drawCrosshairLabelsFlag);
+
         void setVolumeSliceViewingAndModelingTransformations(const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
                                                              const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
                                                              const Plane& plane);
@@ -239,11 +267,22 @@ namespace caret {
                         const Plane& slicePlane,
                         const float sliceCoordinates[3]);
         
-        static void drawSurfaceOutline(const ModelTypeEnum::Enum modelType,
-                                       const Plane& plane,
-                                       VolumeSurfaceOutlineSetModel* outlineSet,
-                                       BrainOpenGLFixedPipeline* fixedPipelineDrawing);
-        
+        static void drawSurfaceOutlineCached(const VolumeMappableInterface* underlayVolume,
+                                             const ModelTypeEnum::Enum modelType,
+                                             const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
+                                             const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                             const float sliceXYZ[3],
+                                             const Plane& plane,
+                                             VolumeSurfaceOutlineSetModel* outlineSet,
+                                             BrainOpenGLFixedPipeline* fixedPipelineDrawing,
+                                             const bool useNegativePolygonOffsetFlag);
+
+        static void drawSurfaceOutlineNotCached(const ModelTypeEnum::Enum modelType,
+                                                const Plane& plane,
+                                                VolumeSurfaceOutlineSetModel* outlineSet,
+                                                BrainOpenGLFixedPipeline* fixedPipelineDrawing,
+                                                const bool useNegativePolygonOffsetFlag);
+
         void drawVolumeSliceFoci(const Plane& plane);
         
         void drawAxesCrosshairs(const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
@@ -257,9 +296,8 @@ namespace caret {
         
         void drawSquare(const float size);
         
-        void drawOrientationAxes(const int viewport[4]);
-        
-        void setOrthographicProjection(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+        void setOrthographicProjection(const AllSliceViewMode allSliceViewMode,
+                                       const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
                                        const int viewport[4]);
         
         void drawOrthogonalSliceVoxels(const float sliceNormalVector[3],
@@ -276,16 +314,16 @@ namespace caret {
                                        const uint8_t sliceOpacity);
         
         void drawOrthogonalSliceVoxelsSingleQuads(const float sliceNormalVector[3],
-                                       const float coordinate[3],
-                                       const float rowStep[3],
-                                       const float columnStep[3],
-                                       const int64_t numberOfColumns,
-                                       const int64_t numberOfRows,
-                                       const std::vector<uint8_t>& sliceRGBA,
-                                       const VolumeMappableInterface* volumeInterface,
-                                       const int32_t volumeIndex,
-                                       const int32_t mapIndex,
-                                       const uint8_t sliceOpacity);
+                                                  const float coordinate[3],
+                                                  const float rowStep[3],
+                                                  const float columnStep[3],
+                                                  const int64_t numberOfColumns,
+                                                  const int64_t numberOfRows,
+                                                  const std::vector<uint8_t>& sliceRGBA,
+                                                  const VolumeMappableInterface* volumeInterface,
+                                                  const int32_t volumeIndex,
+                                                  const int32_t mapIndex,
+                                                  const uint8_t sliceOpacity);
         
         void drawOrthogonalSliceVoxelsQuadIndicesAndStrips(const float sliceNormalVector[3],
                                                            const float coordinate[3],
@@ -302,9 +340,6 @@ namespace caret {
         bool getVoxelCoordinateBoundsAndSpacing(float boundsOut[6],
                                                 float spacingOut[3]);
         
-        void createObliqueTransformationMatrix(const float sliceCoordinates[3],
-                                               Matrix4x4& obliqueTransformationMatrixOut);
-        
         void drawIdentificationSymbols(const Plane& plane);
         
         void addVoxelToIdentification(const int32_t volumeIndex,
@@ -364,14 +399,12 @@ namespace caret {
         
         static const int32_t IDENTIFICATION_INDICES_PER_VOXEL;
         
-        friend class BrainOpenGLVolumeObliqueSliceDrawing;
-        
         // ADD_NEW_MEMBERS_HERE
     };
     
 #ifdef __BRAIN_OPEN_GL_VOLUME_SLICE_DRAWING_DECLARE__
     const int32_t BrainOpenGLVolumeSliceDrawing::IDENTIFICATION_INDICES_PER_VOXEL = 8;
 #endif // __BRAIN_OPEN_GL_VOLUME_SLICE_DRAWING_DECLARE__
-
+    
 } // namespace
 #endif  //__BRAIN_OPEN_GL_VOLUME_SLICE_DRAWING_H__
