@@ -229,6 +229,7 @@ ModelSurfaceMontage::getSelectedSurface(const StructureEnum::Enum structure,
             switch (structure) {
                 case StructureEnum::CEREBELLUM:
                     selectionModels.push_back(smcfm->getCerebellumSurfaceSelectionModel());
+                    break;
                 case StructureEnum::CORTEX_LEFT:
                     selectionModels.push_back(smcfm->getLeftSurfaceSelectionModel());
                     break;
@@ -945,15 +946,14 @@ ModelSurfaceMontage::getSurfaceMontageNumberOfRowsAndColumns(const int32_t tabIn
     numberOfColumnsOut = 1;
     
     ModelSurfaceMontage* nonConstMSM = const_cast<ModelSurfaceMontage*>(this);
-    std::vector<SurfaceMontageViewport*> surfaceMontageViewports;
-    nonConstMSM->getSurfaceMontageViewportsForDrawing(tabIndex,
-                                                      surfaceMontageViewports);
+    std::vector<const SurfaceMontageViewport*> surfaceMontageViewports;
+    nonConstMSM->getSurfaceMontageViewportsForTransformation(tabIndex,
+                                                             surfaceMontageViewports);
     SurfaceMontageViewport::getNumberOfRowsAndColumns(surfaceMontageViewports,
                                                       numberOfRowsOut,
                                                       numberOfColumnsOut);
 }
 
-
 /**
  * Get the montage viewports for drawing by OpenGL.  The montage viewports
  * will be updated prior to returning them.   OpenGL will update
