@@ -109,7 +109,7 @@ logHeadersFromRequest(const QNetworkRequest& request,
         infoText.appendWithNewLine("        Contains no headers");
     }
     
-    CaretLogInfo(infoText);
+    CaretLogFine(infoText);
 }
 
 void
@@ -136,14 +136,25 @@ logHeadersFromReply(const QNetworkReply& reply,
 {
     AString infoText;
     infoText.appendWithNewLine("Reply " + caretHttpRequestToName(caretHttpRequest) + " URL (" + QString::number(caretHttpResponse.m_responseCode) + ") Header: ");
+    bool errorFlag(false);
     if ( ! caretHttpResponse.m_responseCodeValid) {
         infoText.appendWithNewLine("RESPONSE CODE IS NOT VALID.");
+        errorFlag = true;
     }
     const QNetworkReply::NetworkError networkErrorCode = reply.error();
     if (networkErrorCode != QNetworkReply::NoError) {
         infoText.appendWithNewLine("Network Error Code (See QNetworkReply::NetworkError for description): "
-                                   + QString::number(static_cast<int>(networkErrorCode)));
+                                   + QString::number(static_cast<int>(networkErrorCode))
+                                   + "\nDescription: "
+                                   + reply.errorString());
+        errorFlag = true;
     }
+    
+    if (errorFlag) {
+        CaretLogWarning(infoText);
+        return;
+    }
+    
     QList<QByteArray> readHeaderList = reply.rawHeaderList();
     const int numItems = readHeaderList.size();
     if (numItems > 0) {
@@ -160,7 +171,7 @@ logHeadersFromReply(const QNetworkReply& reply,
         infoText.appendWithNewLine("    Contains no headers");
     }
     
-    CaretLogInfo(infoText);
+    CaretLogFine(infoText);
 }
 
 void CaretHttpManager::httpRequest(const CaretHttpRequest &request, CaretHttpResponse &response)
@@ -183,7 +194,7 @@ void CaretHttpManager::httpRequest(const CaretHttpRequest &request, CaretHttpRes
             CaretHttpRequest redirectedRequest = request;
             redirectedRequest.m_url = response.m_redirectionUrl.toString();
 
-            CaretLogInfo("Received and processing redirection request from "
+            CaretLogFine("Received and processing redirection request from "
                            + request.m_url
                            + " to "
                            + redirectedRequest.m_url);
@@ -222,14 +233,14 @@ void CaretHttpManager::httpRequestPrivate(const CaretHttpRequest &request, Caret
         {
             QString unencoded = myCaretMgr->m_authList[i].m_user + ":" + myCaretMgr->m_authList[i].m_pass;
             myRequest.setRawHeader("Authorization", "Basic " + unencoded.toLocal8Bit().toBase64());
-            CaretLogInfo("Found auth for URL " + request.m_url);
+            CaretLogFine("Found auth for URL " + request.m_url);
             have_auth = true;
             break;
         }
     }
     if (!have_auth)
     {
-        CaretLogInfo("NO AUTH FOUND for URL " + request.m_url);
+        CaretLogFine("NO AUTH FOUND for URL " + request.m_url);
     }
     QNetworkReply* myReply = NULL;
 /*
@@ -280,7 +291,7 @@ void CaretHttpManager::httpRequestPrivate(const CaretHttpRequest &request, Caret
 #endif // QT_VERSION
             myRequest.setUrl(myUrl);
             myReply = myQNetMgr->post(myRequest, postData);
-            CaretLogInfo("POST ARGUMENTS URL: " + myUrl.toString());
+            CaretLogFine("POST ARGUMENTS URL: " + myUrl.toString());
         }
         break;
     case POST_FILE:
@@ -300,7 +311,7 @@ void CaretHttpManager::httpRequestPrivate(const CaretHttpRequest &request, Caret
 #endif // QT_VERSION
                 myRequest.setUrl(myUrl);
                 myReply = myQNetMgr->post(myRequest, postUploadFile);
-                CaretLogInfo("POST FILE URL: " + myUrl.toString());
+                CaretLogFine("POST FILE URL: " + myUrl.toString());
             }
             else {
                 
@@ -320,7 +331,7 @@ void CaretHttpManager::httpRequestPrivate(const CaretHttpRequest &request, Caret
         myUrl.setQuery(myUrlQuery);
 #endif // QT_VERSION
         myRequest.setUrl(myUrl);
-        CaretLogInfo("GET URL: " + myUrl.toString());
+        CaretLogFine("GET URL: " + myUrl.toString());
         myReply = myQNetMgr->get(myRequest);
         break;
     case HEAD:
@@ -336,7 +347,7 @@ void CaretHttpManager::httpRequestPrivate(const CaretHttpRequest &request, Caret
         myUrl.setQuery(myUrlQuery);
 #endif // QT_VERSION
         myRequest.setUrl(myUrl);
-        CaretLogInfo("HEAD URL: " + myUrl.toString());
+        CaretLogFine("HEAD URL: " + myUrl.toString());
         myReply = myQNetMgr->head(myRequest);
         break;
     };
@@ -354,6 +365,7 @@ void CaretHttpManager::httpRequestPrivate(const CaretHttpRequest &request, Caret
     response.m_responseCode = -1;
     response.m_responseCodeValid = false;
     response.m_headers.clear();
+    response.m_errorMessage.clear();
     const QVariant responseCodeVariant = myReply->attribute(QNetworkRequest::HttpStatusCodeAttribute);
     if ( ! responseCodeVariant.isNull()) {
         response.m_responseCode = myReply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
@@ -384,6 +396,15 @@ void CaretHttpManager::httpRequestPrivate(const CaretHttpRequest &request, Caret
                             request,
                             response);
     }
+    if (response.m_responseCode != 200) {
+        AString s("QNetworkReply::NetworkError Code="
+                  + AString::number((int)myReply->error())
+                  + ", Error="
+                  + myReply->errorString()
+                  + "\nURL="
+                  + myUrl.toString(QUrl::None));
+        response.m_errorMessage = s;
+    }
     
     QByteArray myBody = myReply->readAll();
     int64_t mySize = myBody.size();
@@ -400,7 +421,7 @@ void CaretHttpManager::setAuthentication(const AString& url, const AString& user
 {
     CaretHttpManager* myCaretMgr = getHttpManager();
     AString myServerString = getServerString(url);
-    CaretLogInfo("Setting auth for server " + myServerString);
+    CaretLogFine("Setting auth for server " + myServerString);
     for (int i = 0; i < (int)myCaretMgr->m_authList.size(); ++i)
     {
         if (myServerString == myCaretMgr->m_authList[i].m_serverString)
