@@ -488,7 +488,8 @@ FileInformation::getCanonicalPath() const
 AString
 FileInformation::getFileExtension() const
 {
-    const std::vector<AString> workbenchExtensions = DataFileTypeEnum::getFilesExtensionsForEveryFile();
+    const bool includeNonWritableFileTypesFlag(true);
+    const std::vector<AString> workbenchExtensions = DataFileTypeEnum::getFilesExtensionsForEveryFile(includeNonWritableFileTypesFlag);
 
     for (std::vector<AString>::const_iterator extIter = workbenchExtensions.begin();
          extIter != workbenchExtensions.end();
@@ -819,4 +820,25 @@ FileInformation::fileSizeToStandardUnits(const int64_t numberOfBytes)
     return sizeString;
 }
 
+/**
+ * Cleans the path.  First, replaces any backward slashes with forward slashes
+ * and then calls QDir::cleanPath().   Note that QDir::cleanPath() only converts
+ * backward slashes to forward slashes when the native separator is a backward
+ * slash (on Windows).   Thus, if a path on Mac or Linux contains a backward
+ * slash, QDir::cleanPath() will not convert to a forward slash.
+ *
+ * @param path
+ *     This input path.
+ * @return
+ *     Path after cleaning.
+ */
+AString
+FileInformation::cleanPath(const AString& path)
+{
+    AString pathOut(path);
+    pathOut.replace(QLatin1Char('\\'), QLatin1Char('/'));
+    pathOut = QDir::cleanPath(pathOut);
+    return pathOut;
+}
+
 
