@@ -1385,41 +1385,74 @@ CaretMappableDataFile::isOnePaletteUsedForAllMaps() const
 PaletteNormalizationModeEnum::Enum
 CaretMappableDataFile::getPaletteNormalizationMode() const
 {
-    PaletteNormalizationModeEnum::Enum modeValue = PaletteNormalizationModeEnum::NORMALIZATION_SELECTED_MAP_DATA;
-    
     const AString textValue = getFileMetaData()->get(GiftiMetaDataXmlElements::METADATA_PALETTE_NORMALIZATION_MODE);
     bool validFlag = false;
-    modeValue = PaletteNormalizationModeEnum::fromName(textValue, &validFlag);
+    PaletteNormalizationModeEnum::Enum modeValue = PaletteNormalizationModeEnum::fromName(textValue, &validFlag);
     
     if ( ! validFlag) {
-        std::vector<PaletteNormalizationModeEnum::Enum> validModes;
-        getPaletteNormalizationModesSupported(validModes);
-        
-        if ( ! validModes.empty()) {
-            CaretAssertVectorIndex(validModes, 0);
-            modeValue = validModes[0];
-        }
-        else {
-            modeValue = PaletteNormalizationModeEnum::NORMALIZATION_SELECTED_MAP_DATA;
-        }
+        modeValue = getDefaultPaletteNormalizationMode();
     }
     
+    ensurePaletteNormalizationModeIsSupported(modeValue);
+    
     return modeValue;
 }
 
 /**
- * Set the palette normalization mode for the file.
+ * Set the palette normalization mode for the file.  If the mode is not supported by the
+ * file, the mode is not changed.
  *
  * @param mode
  *     New value for palette normalization mode.
  */
 void
-CaretMappableDataFile::setPaletteNormalizationMode(const PaletteNormalizationModeEnum::Enum mode)
+CaretMappableDataFile::setPaletteNormalizationMode(const PaletteNormalizationModeEnum::Enum modeIn)
 {
+    PaletteNormalizationModeEnum::Enum mode = modeIn;
+    ensurePaletteNormalizationModeIsSupported(mode);
     getFileMetaData()->set(GiftiMetaDataXmlElements::METADATA_PALETTE_NORMALIZATION_MODE,
                            PaletteNormalizationModeEnum::toName(mode));
 }
 
+/**
+ * Ensure that the given palette normalization mode is supported by the instance of this file.
+ * If not supported, it is changed to that returned by getDefaultPaletteNormalizationMode().
+ *
+ * @param modeInOut
+ *     Normalization mode that may be changed to a supported mode.
+ */
+void
+CaretMappableDataFile::ensurePaletteNormalizationModeIsSupported(PaletteNormalizationModeEnum::Enum& modeInOut) const
+{
+    std::vector<PaletteNormalizationModeEnum::Enum> validModes;
+    getPaletteNormalizationModesSupported(validModes);
+    
+    if (std::find(validModes.begin(),
+                  validModes.end(),
+                  modeInOut) == validModes.end()) {
+        modeInOut = getDefaultPaletteNormalizationMode();
+    }
+}
+
+/**
+ * @return The default palette normalization mode for this file which is always
+ * the first mode in the supported modes from getPaletteNormalizationModesSupported().
+ */
+PaletteNormalizationModeEnum::Enum
+CaretMappableDataFile::getDefaultPaletteNormalizationMode() const
+{
+    PaletteNormalizationModeEnum::Enum mode = PaletteNormalizationModeEnum::NORMALIZATION_SELECTED_MAP_DATA;
+    
+    std::vector<PaletteNormalizationModeEnum::Enum> validModes;
+    getPaletteNormalizationModesSupported(validModes);
+    if ( ! validModes.empty()) {
+        CaretAssertVectorIndex(validModes, 0);
+        mode = validModes[0];
+    }
+    
+    return mode;
+}
+
 /**
  * @return Is the data in the file mapped to colors using
  * Red, Green, Blue, Alpha values.
@@ -1440,3 +1473,51 @@ const CiftiXML CaretMappableDataFile::getCiftiXML() const
     return CiftiXML();
 }
 
+/**
+ * Get the identification information for a surface node in the given maps.
+ *
+ * @param mapIndices
+ *    Indices of maps for which identification information is requested.
+ * @param structure
+ *    Structure of the surface.
+ * @param nodeIndex
+ *    Index of the node.
+ * @param numberOfNodes
+ *    Number of nodes in the surface.
+ * @param textOut
+ *    Output containing identification information.
+ */
+bool
+CaretMappableDataFile::getSurfaceNodeIdentificationForMaps(const std::vector<int32_t>& /*mapIndices*/,
+                                                           const StructureEnum::Enum /*structure*/,
+                                                           const int /*nodeIndex*/,
+                                                           const int32_t /*numberOfNodes*/,
+                                                           AString& textOut) const
+{
+    textOut.clear();
+    return false;
+}
+
+/**
+ * Get the identification information for a surface node in the given maps.
+ *
+ * @param mapIndices
+ *    Indices of maps for which identification information is requested.
+ * @param xyz
+ *     Coordinate of voxel.
+ * @param ijkOut
+ *     Voxel indices of value.
+ * @param textOut
+ *    Output containing identification information.
+ */
+bool
+CaretMappableDataFile::getVolumeVoxelIdentificationForMaps(const std::vector<int32_t>& /*mapIndices*/,
+                                                           const float* /*xyz[3]*/,
+                                                           int64_t* /*ijkOut[3]*/,
+                                                           AString& textOut) const
+{
+    textOut.clear();
+    return false;
+}
+
+
