@@ -397,18 +397,9 @@ namespace caret {
          */
         virtual void getPaletteNormalizationModesSupported(std::vector<PaletteNormalizationModeEnum::Enum>& modesSupportedOut) const = 0;
         
-        /**
-         * @return The palette normalization mode for the file.
-         */
-        virtual PaletteNormalizationModeEnum::Enum getPaletteNormalizationMode() const;
+        PaletteNormalizationModeEnum::Enum getPaletteNormalizationMode() const;
 
-        /**
-         * Set the palette normalization mode for the file.
-         *
-         * @param mode
-         *     New value for palette normalization mode.
-         */
-        virtual void setPaletteNormalizationMode(const PaletteNormalizationModeEnum::Enum mode);
+        void setPaletteNormalizationMode(const PaletteNormalizationModeEnum::Enum mode);
         
         /**
          * Update coloring for all maps.
@@ -513,6 +504,20 @@ namespace caret {
          */
         virtual BrainordinateMappingMatch getBrainordinateMappingMatch(const CaretMappableDataFile* mapFile) const = 0;
         
+        virtual bool getSurfaceNodeIdentificationForMaps(const std::vector<int32_t>& mapIndices,
+                                                         const StructureEnum::Enum structure,
+                                                         const int nodeIndex,
+                                                         const int32_t numberOfNodes,
+                                                         AString& textOut) const;
+        
+        virtual bool getVolumeVoxelIdentificationForMaps(const std::vector<int32_t>& mapIndices,
+                                                         const float xyz[3],
+                                                         int64_t ijkOut[3],
+                                                         AString& textOut) const;
+        
+
+        void updateAfterFileDataChanges();
+        
     protected:
         CaretMappableDataFile(const CaretMappableDataFile&);
 
@@ -522,8 +527,6 @@ namespace caret {
         
         void helpGetSupportedLineSeriesChartDataTypes(std::vector<ChartOneDataTypeEnum::Enum>& chartDataTypesOut) const;
         
-        void updateAfterFileDataChanges();
-        
         virtual void saveFileDataToScene(const SceneAttributes* sceneAttributes,
                                          SceneClass* sceneClass);
         
@@ -534,6 +537,10 @@ namespace caret {
         
         void copyCaretMappableDataFile(const CaretMappableDataFile&);
         
+        void ensurePaletteNormalizationModeIsSupported(PaletteNormalizationModeEnum::Enum& modeInOut) const;
+        
+        PaletteNormalizationModeEnum::Enum getDefaultPaletteNormalizationMode() const;
+        
         bool isPaletteColorMappingEqualForAllMaps() const;
         
         void updateMapThresholdFileSelectionModels();
