@@ -89,6 +89,8 @@ m_validRowColumnSelectionDimensions(validRowColumnSelectionDimensions)
         switch (ciftiMapFile->getDataFileType()) {
             case DataFileTypeEnum::ANNOTATION:
                 break;
+            case DataFileTypeEnum::ANNOTATION_TEXT_SUBSTITUTION:
+                break;
             case DataFileTypeEnum::BORDER:
                 break;
             case DataFileTypeEnum::CONNECTIVITY_DENSE:
@@ -132,6 +134,8 @@ m_validRowColumnSelectionDimensions(validRowColumnSelectionDimensions)
                 break;
             case DataFileTypeEnum::METRIC:
                 break;
+            case DataFileTypeEnum::METRIC_DYNAMIC:
+                break;
             case DataFileTypeEnum::PALETTE:
                 break;
             case DataFileTypeEnum::RGBA:
@@ -146,6 +150,8 @@ m_validRowColumnSelectionDimensions(validRowColumnSelectionDimensions)
                 break;
             case DataFileTypeEnum::VOLUME:
                 break;
+            case DataFileTypeEnum::VOLUME_DYNAMIC:
+                break;
         }
         
         bool hasColumnParcelsFlag  = false;
