@@ -630,7 +630,7 @@ SceneFile::readFile(const AString& filenameIn)
     else {
         SceneFileSaxReader saxReader(this,
                                      filename);
-        std::auto_ptr<XmlSaxParser> parser(XmlSaxParser::createXmlParser());
+        std::unique_ptr<XmlSaxParser> parser(XmlSaxParser::createXmlParser());
         try {
             parser->parseFile(filename, &saxReader);
         }
