@@ -32,6 +32,7 @@
 #include "DataFileTypeEnum.h"
 #include "EventListenerInterface.h"
 #include "SceneableInterface.h"
+#include "TileTabsGridModeEnum.h"
 
 class QAction;
 class QActionGroup;
@@ -45,6 +46,7 @@ namespace caret {
     class BrainOpenGLWidget;
     class BrowserWindowContent;
     class BrowserTabContent;
+    class EventTileTabsConfigurationModification;
     class PlainTextStringBuilder;
     class SceneClassAssistant;
     class TileTabsConfiguration;
@@ -115,10 +117,6 @@ namespace caret {
         virtual void restoreFromScene(const SceneAttributes* sceneAttributes,
                                       const SceneClass* sceneClass);
 
-        TileTabsConfiguration* getSelectedTileTabsConfiguration();
-        
-        void setSelectedTileTabsConfiguration(TileTabsConfiguration* configuration);
-        
         void resetGraphicsWidgetMinimumSize();
         
         void setGraphicsWidgetFixedSize(const int32_t width,
@@ -152,8 +150,13 @@ namespace caret {
         
         bool isOpenGLContextSharingValid() const;
 
+        AString getTileTabsConfigurationLabelText(const TileTabsGridModeEnum::Enum configurationMode,
+                                                  const bool includeRowsAndColumns) const;
+
         void resizeDockWidgets(const QList<QDockWidget *> &docks, const QList<int> &sizes, Qt::Orientation orientation);
         
+        static void setEnableMacDuplicateMenuBar(bool status);
+        
     protected:
         void closeEvent(QCloseEvent* event);
         void keyPressEvent(QKeyEvent* event);
@@ -168,6 +171,7 @@ namespace caret {
         void processDataFileOpen();
         void processManageSaveLoadedFiles();
         void processCaptureImage();
+        void processMovieRecording();
         void processRecordMovie();
         void processEditPreferences();
         void processCloseAllFiles();
@@ -181,6 +185,11 @@ namespace caret {
         void processShowIdentifyBrainordinateDialog();
         void processGapsAndMargins();
         
+        void processViewTileTabsLoadUserConfigurationMenuAboutToShow();
+        void processViewTileTabsLoadUserConfigurationMenuItemTriggered(QAction* action);
+        void processViewTileTabsAutomaticCustomTriggered(QAction* action);
+        
+
         void processMoveOverlayToolBoxToLeft();
         void processMoveOverlayToolBoxToBottom();
         void processMoveOverlayToolBoxToFloat();
@@ -208,9 +217,6 @@ namespace caret {
         void processDataMenuAboutToShow();
         void processViewMenuAboutToShow();
         
-        void processTileTabsMenuAboutToBeDisplayed();
-        void processTileTabsMenuSelection(QAction*);
-        
         void processSurfaceMenuInformation();
         void processSurfaceMenuPrimaryAnatomical();
         
@@ -222,6 +228,7 @@ namespace caret {
         void processReportWorkbenchBug();
         
         void processShowSurfacePropertiesDialog();
+        void processShowVolumePropertiesDialog();
         
         void processDevelopGraphicsTiming();
         
@@ -290,6 +297,7 @@ namespace caret {
         QMenu* createMenuView();
         QMenu* createMenuViewMoveOverlayToolBox();
         QMenu* createMenuViewMoveFeaturesToolBox();
+        QMenu* createMenuViewTileTabsLoadUserConfiguration();
         QMenu* createMenuConnect();
         QMenu* createMenuData();
         QMenu* createMenuSurface();
@@ -318,6 +326,8 @@ namespace caret {
         void lockAllTabAspectRatios(const bool checked);
         void updateActionsForLockingAspectRatios();
         
+        void processToolBarLockWindowAndAllTabAspectsRatios(bool checked);
+        
         float getAspectRatioFromDialog(const AspectRatioMode aspectRatioMode,
                                        const QString& title,
                                        const float aspectRatio,
@@ -325,6 +335,8 @@ namespace caret {
         
         void saveBrowserWindowContentForScene();
         
+        void modifyTileTabsConfiguration(EventTileTabsConfigurationModification* modEvent);
+
         /** Index of this window */
         const int32_t m_browserWindowIndex;
         
@@ -359,6 +371,8 @@ namespace caret {
         
         QAction* m_captureImageAction;
 
+        QAction* m_movieRecordingAction;
+        
         QAction* m_recordMovieAction;
         
         QAction* m_preferencesAction;
@@ -369,12 +383,15 @@ namespace caret {
         
         QMenu* m_viewMoveFeaturesToolBoxMenu;
         QMenu* m_viewMoveOverlayToolBoxMenu;
+        QMenu* m_viewTileTabsLoadUserConfigurationMenu;
         
         QAction* m_viewFullScreenAction;
         QAction* m_viewTileTabsAction;
         
-        QMenu* m_tileTabsMenu;
-        QAction* m_createAndEditTileTabsAction;
+        QAction* m_viewTileTabsConfigurationDialogAction;
+        QAction* m_viewAutomaticTileTabsConfigurationAction;
+        QAction* m_viewCustomTileTabsConfigurationAction;
+        std::vector<std::pair<QAction*, TileTabsConfiguration*>> m_viewCustomTileTabsConfigurationActions;
         
         QAction* m_gapsAndMarginsAction;
         
@@ -434,9 +451,6 @@ namespace caret {
         BrainBrowserWindowOrientedToolBox* m_overlayActiveToolBox;
         BrainBrowserWindowOrientedToolBox* m_featuresToolBox;
         
-        AString m_selectedTileTabsConfigurationUniqueIdentifier;
-        TileTabsConfiguration* m_defaultTileTabsConfiguration;
-        
         static AString s_previousOpenFileNameFilter;
         static AString s_previousOpenFileDirectory;
         static QByteArray s_previousOpenFileGeometry;
@@ -458,6 +472,13 @@ namespace caret {
         static int32_t s_sceneFileFirstWindowY;
         
         static std::set<BrainBrowserWindow*> s_brainBrowserWindows;
+        
+        QString m_objectNamePrefix;
+        
+        bool m_keyEventProcessingFlag = false;
+        
+        static bool s_enableMacDuplicateMenuBarFlag;
+        
     };
 #ifdef __BRAIN_BROWSER_WINDOW_DECLARE__
     std::set<BrainBrowserWindow*> BrainBrowserWindow::s_brainBrowserWindows;
@@ -468,6 +489,8 @@ namespace caret {
     bool BrainBrowserWindow::s_firstWindowFlag = true;
     int32_t BrainBrowserWindow::s_sceneFileFirstWindowX = -1;
     int32_t BrainBrowserWindow::s_sceneFileFirstWindowY = -1;
+
+    bool BrainBrowserWindow::s_enableMacDuplicateMenuBarFlag = false;
 #endif // __BRAIN_BROWSER_WINDOW_DECLARE__
     
 }
