@@ -41,12 +41,16 @@ using namespace caret;
 #include "CaretLogger.h"
 #include "CiftiFiberTrajectoryFile.h"
 #include "CaretMappableDataFile.h"
+#include "ChartableTwoFileDelegate.h"
+#include "ChartableTwoFileLineSeriesChart.h"
+#include "ChartableTwoFileMatrixChart.h"
 #include "CiftiConnectivityMatrixDataFileManager.h"
 #include "CiftiFiberTrajectoryManager.h"
 #include "CiftiMappableConnectivityMatrixDataFile.h"
 #include "CaretMappableDataFileAndMapSelectionModel.h"
 #include "CaretMappableDataFileAndMapSelectorObject.h"
 #include "CiftiParcelSelectionComboBox.h"
+#include "CiftiScalarDataSeriesFile.h"
 #include "EventGraphicsUpdateAllWindows.h"
 #include "EventIdentificationHighlightLocation.h"
 #include "EventManager.h"
@@ -68,6 +72,7 @@ using namespace caret;
 #include "WuQFactory.h"
 #include "WuQGroupBoxExclusiveWidget.h"
 #include "WuQMessageBox.h"
+#include "WuQSpinBox.h"
 #include "WuQtUtilities.h"
 
 #include <limits>
@@ -93,7 +98,9 @@ IdentifyBrainordinateDialog::IdentifyBrainordinateDialog(QWidget* parent)
      */
     std::vector<DataFileTypeEnum::Enum> allDataFileTypes;
     DataFileTypeEnum::getAllEnums(allDataFileTypes,
-                                  DataFileTypeEnum::OPTIONS_INCLUDE_CONNECTIVITY_DENSE_DYNAMIC);
+                                  (DataFileTypeEnum::OPTIONS_INCLUDE_CONNECTIVITY_DENSE_DYNAMIC
+                                   | DataFileTypeEnum::OPTIONS_INCLUDE_METRIC_DENSE_DYNAMIC
+                                   | DataFileTypeEnum::OPTIONS_INCLUDE_VOLUME_DENSE_DYNAMIC));
     
     std::vector<DataFileTypeEnum::Enum> supportedCiftiRowFileTypes;
     std::vector<DataFileTypeEnum::Enum> supportedLabelFileTypes;
@@ -109,6 +116,8 @@ IdentifyBrainordinateDialog::IdentifyBrainordinateDialog(QWidget* parent)
         switch (dataFileType) {
             case DataFileTypeEnum::ANNOTATION:
                 break;
+            case DataFileTypeEnum::ANNOTATION_TEXT_SUBSTITUTION:
+                break;
             case DataFileTypeEnum::BORDER:
                 break;
             case DataFileTypeEnum::CONNECTIVITY_DENSE:
@@ -151,6 +160,7 @@ IdentifyBrainordinateDialog::IdentifyBrainordinateDialog(QWidget* parent)
                 ciftiRowFlag = true;
                 break;
             case DataFileTypeEnum::CONNECTIVITY_SCALAR_DATA_SERIES:
+                ciftiRowFlag = true;
                 break;
             case DataFileTypeEnum::FOCI:
                 break;
@@ -161,6 +171,8 @@ IdentifyBrainordinateDialog::IdentifyBrainordinateDialog(QWidget* parent)
                 break;
             case DataFileTypeEnum::METRIC:
                 break;
+            case DataFileTypeEnum::METRIC_DYNAMIC:
+                break;
             case DataFileTypeEnum::PALETTE:
                 break;
             case DataFileTypeEnum::RGBA:
@@ -175,6 +187,8 @@ IdentifyBrainordinateDialog::IdentifyBrainordinateDialog(QWidget* parent)
                 break;
             case DataFileTypeEnum::VOLUME:
                 break;
+            case DataFileTypeEnum::VOLUME_DYNAMIC:
+                break;
         }
         
         if (parcelSourceDimension != PARCEL_SOURCE_INVALID_DIMENSION) {
@@ -206,7 +220,7 @@ IdentifyBrainordinateDialog::IdentifyBrainordinateDialog(QWidget* parent)
     m_ciftiParcelWidget = createCiftiParcelWidget();
     
     m_widgetBox = new WuQGroupBoxExclusiveWidget();
-    m_widgetBox->addWidget(m_ciftiRowWidget, "Identify Brainordinate from CIFTI File Row");
+    m_widgetBox->addWidget(m_ciftiRowWidget, "Identify from CIFTI File Row");
     m_widgetBox->addWidget(m_ciftiParcelWidget, "Identify CIFTI File Parcel");
     m_widgetBox->addWidget(m_labelFileWidgets.m_widget, "Identify Label");
     m_widgetBox->addWidget(m_surfaceVertexWidget, "Identify Surface Vertex");
@@ -331,11 +345,12 @@ IdentifyBrainordinateDialog::createCiftiRowWidget(const std::vector<DataFileType
     m_ciftiRowFileComboBox = new CaretDataFileSelectionComboBox(this);
     
     m_ciftiRowFileIndexLabel = new QLabel("Row Index");
-    m_ciftiRowFileIndexSpinBox = WuQFactory::newSpinBoxWithMinMaxStep(CiftiMappableDataFile::getCiftiFileRowColumnIndexBaseForGUI(),
-                                                                      std::numeric_limits<int>::max(),
-                                                                      1);
-    QObject::connect(m_ciftiRowFileIndexSpinBox, SIGNAL(valueChanged(int)),
+    m_ciftiRowFileIndexSpinBox = new WuQSpinBox();
+    m_ciftiRowFileIndexSpinBox->setMinimum(1);
+    m_ciftiRowFileIndexSpinBox->setMaximum(std::numeric_limits<int>::max());
+    QObject::connect(m_ciftiRowFileIndexSpinBox, SIGNAL(signalReturnPressed()),
                      this, SLOT(apply()));
+
     m_ciftiRowFileIndexSpinBox->setFixedWidth(INDEX_SPIN_BOX_WIDTH);
     switch (CiftiMappableDataFile::getCiftiFileRowColumnIndexBaseForGUI()) {
         case 0:
@@ -758,6 +773,7 @@ IdentifyBrainordinateDialog::processCiftiRowWidget(AString& errorMessageOut)
     if (dataFile != NULL) {
         CiftiMappableDataFile*    ciftiMapFile  = dynamic_cast<CiftiMappableDataFile*>(dataFile);
         CiftiFiberTrajectoryFile* ciftiTrajFile = dynamic_cast<CiftiFiberTrajectoryFile*>(dataFile);
+        CiftiScalarDataSeriesFile* ciftiSdsFile = dynamic_cast<CiftiScalarDataSeriesFile*>(dataFile);
         
         const int32_t selectedCiftiRowIndex = m_ciftiRowFileIndexSpinBox->value() - m_ciftiRowFileIndexSpinBox->minimum();
         
@@ -769,75 +785,101 @@ IdentifyBrainordinateDialog::processCiftiRowWidget(AString& errorMessageOut)
             int64_t voxelIJK[3];
             float voxelXYZ[3];
             bool voxelValid = false;
-            if (ciftiMapFile != NULL) {
-                ciftiMapFile->getBrainordinateFromRowIndex(selectedCiftiRowIndex,
-                                                           surfaceStructure,
-                                                           surfaceNodeIndex,
-                                                           surfaceNumberOfNodes,
-                                                           surfaceNodeValid,
-                                                           voxelIJK,
-                                                           voxelXYZ,
-                                                           voxelValid);
-            }
-            else if (ciftiTrajFile != NULL) {
-                ciftiTrajFile->getBrainordinateFromRowIndex(selectedCiftiRowIndex,
-                                                            surfaceStructure,
-                                                            surfaceNodeIndex,
-                                                            surfaceNumberOfNodes,
-                                                            surfaceNodeValid,
-                                                            voxelIJK,
-                                                            voxelXYZ,
-                                                            voxelValid);
+            
+            if (ciftiSdsFile != NULL) {
+                ChartableTwoFileDelegate* chartDelegate = ciftiSdsFile->getChartingDelegate();
+                CaretAssert(chartDelegate);
+                ChartableTwoFileMatrixChart* matrixChart = chartDelegate->getMatrixCharting();
+                ChartableTwoFileLineSeriesChart* lineSeriesChart = chartDelegate->getLineSeriesCharting();
+                if (matrixChart != NULL) {
+                    int32_t rowCount(0), columnCount(0);
+                    matrixChart->getMatrixDimensions(rowCount,
+                                                     columnCount);
+                    if (selectedCiftiRowIndex < rowCount) {
+                        const int32_t tabIndex = 0; // selections are same in all tabs
+                        matrixChart->setSelectedRowColumnIndex(tabIndex,
+                                                               selectedCiftiRowIndex);
+                        if (lineSeriesChart) {
+                            lineSeriesChart->loadDataForRowOrColumn(tabIndex,
+                                                                    selectedCiftiRowIndex);
+                        }
+                        
+                        EventManager::get()->sendEvent(EventUserInterfaceUpdate().getPointer());
+                        EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+                    }
+                }
             }
             else {
-                errorMessageOut = "Neither CIFTI Mappable nor CIFTI Trajectory file.  Has new file type been added?";
-            }
-            
-            
-            if (surfaceNodeValid) {
-                SelectionItemSurfaceNode* surfaceID = selectionManager->getSurfaceNodeIdentification();
-                const Surface* surface = brain->getPrimaryAnatomicalSurfaceForStructure(surfaceStructure);
-                if (surface != NULL) {
-                    if ((surfaceNodeIndex >= 0)
-                        && (surfaceNodeIndex < surface->getNumberOfNodes())) {
-                        surfaceID->setSurface(const_cast<Surface*>(surface));
-                        surfaceID->setBrain(brain);
-                        const float* xyz = surface->getCoordinate(surfaceNodeIndex);
-                        const double doubleXYZ[3] = { xyz[0], xyz[1], xyz[2] };
-                        surfaceID->setModelXYZ(doubleXYZ);
-                        surfaceID->setNodeNumber(surfaceNodeIndex);
-                        
-                        GuiManager::get()->processIdentification(-1, // invalid tab index
-                                                                 selectionManager,
-                                                                 this);
-                    }
-                    else {
-                        errorMessageOut = ("Surface vertex index "
-                                        + AString::number(surfaceNodeIndex)
-                                        + " is not valid for surface "
-                                        + surface->getFileNameNoPath());
-                    }
+                if (ciftiMapFile != NULL) {
+                    ciftiMapFile->getBrainordinateFromRowIndex(selectedCiftiRowIndex,
+                                                               surfaceStructure,
+                                                               surfaceNodeIndex,
+                                                               surfaceNumberOfNodes,
+                                                               surfaceNodeValid,
+                                                               voxelIJK,
+                                                               voxelXYZ,
+                                                               voxelValid);
                 }
-                else{
-                    errorMessageOut = ("No surfaces are loaded for structure "
-                                    + StructureEnum::toGuiName(surfaceStructure));
+                else if (ciftiTrajFile != NULL) {
+                    ciftiTrajFile->getBrainordinateFromRowIndex(selectedCiftiRowIndex,
+                                                                surfaceStructure,
+                                                                surfaceNodeIndex,
+                                                                surfaceNumberOfNodes,
+                                                                surfaceNodeValid,
+                                                                voxelIJK,
+                                                                voxelXYZ,
+                                                                voxelValid);
+                }
+                else {
+                    errorMessageOut = "Neither CIFTI Mappable nor CIFTI Trajectory file.  Has new file type been added?";
                 }
                 
-            }
-            else if (voxelValid) {
-                SelectionItemVoxel* voxelID = selectionManager->getVoxelIdentification();
-                voxelID->setBrain(brain);
-                voxelID->setEnabledForSelection(true);
-                voxelID->setVoxelIdentification(brain,
-                                                ciftiMapFile,
-                                                voxelIJK,
-                                                0.0);
-                const double doubleXYZ[3] = { voxelXYZ[0], voxelXYZ[1], voxelXYZ[2] };
-                voxelID->setModelXYZ(doubleXYZ);
                 
-                GuiManager::get()->processIdentification(-1, // invalid tab index
-                                                         selectionManager,
-                                                         this);
+                if (surfaceNodeValid) {
+                    SelectionItemSurfaceNode* surfaceID = selectionManager->getSurfaceNodeIdentification();
+                    const Surface* surface = brain->getPrimaryAnatomicalSurfaceForStructure(surfaceStructure);
+                    if (surface != NULL) {
+                        if ((surfaceNodeIndex >= 0)
+                            && (surfaceNodeIndex < surface->getNumberOfNodes())) {
+                            surfaceID->setSurface(const_cast<Surface*>(surface));
+                            surfaceID->setBrain(brain);
+                            const float* xyz = surface->getCoordinate(surfaceNodeIndex);
+                            const double doubleXYZ[3] = { xyz[0], xyz[1], xyz[2] };
+                            surfaceID->setModelXYZ(doubleXYZ);
+                            surfaceID->setNodeNumber(surfaceNodeIndex);
+                            
+                            GuiManager::get()->processIdentification(-1, // invalid tab index
+                                                                     selectionManager,
+                                                                     this);
+                        }
+                        else {
+                            errorMessageOut = ("Surface vertex index "
+                                               + AString::number(surfaceNodeIndex)
+                                               + " is not valid for surface "
+                                               + surface->getFileNameNoPath());
+                        }
+                    }
+                    else{
+                        errorMessageOut = ("No surfaces are loaded for structure "
+                                           + StructureEnum::toGuiName(surfaceStructure));
+                    }
+                    
+                }
+                else if (voxelValid) {
+                    SelectionItemVoxel* voxelID = selectionManager->getVoxelIdentification();
+                    voxelID->setBrain(brain);
+                    voxelID->setEnabledForSelection(true);
+                    voxelID->setVoxelIdentification(brain,
+                                                    ciftiMapFile,
+                                                    voxelIJK,
+                                                    0.0);
+                    const double doubleXYZ[3] = { voxelXYZ[0], voxelXYZ[1], voxelXYZ[2] };
+                    voxelID->setModelXYZ(doubleXYZ);
+                    
+                    GuiManager::get()->processIdentification(-1, // invalid tab index
+                                                             selectionManager,
+                                                             this);
+                }
             }
         }
         catch (const DataFileException& dfe) {
