@@ -115,6 +115,8 @@ m_newAnnotationCreatedByContextMenu(NULL)
             case AnnotationCoordinateSpaceEnum::CHART:
                 threeDimCoordFlag = true;
                 break;
+            case AnnotationCoordinateSpaceEnum::SPACER:
+                break;
             case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
                 threeDimCoordFlag = true;
                 break;
@@ -540,7 +542,7 @@ UserInputModeAnnotationsContextMenu::createDuplicateTabSpaceAnnotationMenu()
     if (menuValidFlag) {
         for (BrowserTabContent* tabContent : allTabs) {
             if (tabContent->getTabNumber() != m_annotation->getTabIndex()) {
-                QAction* action = menu->addAction(tabContent->getName());
+                QAction* action = menu->addAction(tabContent->getTabName());
                 action->setData((int)tabContent->getTabNumber());
             }
         }
